{-----------------------------------------------------------------------------
 Unit Name: frmFrameRecon
 Author:    Giandomenico De Sanctis @ 5/2009
 Date:      27-mag-2009
 Purpose:   OpenCV demo program, track in realtime a black frame on a webcam
        image calculating perspective parameters (intrinsic matrix), and
        reproducing undistorted bird-eye view of frame
        The program use the FastMM4 fast memory manager .
-----------------------------------------------------------------------------}

unit frmFrameRecon;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,  StdCtrls, ComCtrls, ExtCtrls, Math,
  IPL, OpenCV, jpeg, Spin;


const

      DEG_TO_RAD = 2.0 * PI / 360.0;
type
  TfrmFrameRecon = class(TForm)
    Panel2: TPanel;
    Panel1: TPanel;
    btSettings: TButton;
    Label1: TLabel;
    sePar1: TSpinEdit;
    sePar2: TSpinEdit;
    cbPar3: TComboBox;
    Label2: TLabel;
    sePolyPar1: TSpinEdit;
    sePolyPar2: TSpinEdit;
    Label4: TLabel;
    sePerimRatioDiff: TSpinEdit;
    cbCanny: TCheckBox;
    cbDisplayCont: TCheckBox;
    lbDist: TLabel;
    Image1: TImage;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btSettingsClick(Sender: TObject);
  private
    { Private declarations }
    procedure extractContours(const origImg, image: PIplImage; const storage: PCvMemStorage;
                                var contList: PCvSeq);
    procedure drawFrame(const origImg, image: PIplImage);
    procedure drawPerspective(image: PIplImage);
  public
    { Public declarations }
  end;



var
  fFrameRecon: TfrmFRameRecon;

{-----------------------}
image: pIplImage = 0;
imageDiv: PIplImage;
outImg: pIplImage = 0;
extractedFrame: PIplImage;
gray: PIplImage;
pgray, pcontor: PIplImage;

           
intrinsic: PCvMat;
distortion: PCvMat; 
obj, pro: PCvMat;
rot, tra: PCvMat;
mapMatrix: PCvMat;
origQuad, quad: array[0..3] of CvPoint2D32f;


{-----------------------}

capture: PCvCapture;
frame: PIplImage;
selcam: longint = 0;
bmp: TBitmap;
cont: integer;


{*************************************************************************}
implementation

{$R *.dfm}


procedure main_cycle(frame: PIplImage);
var
cs: CvSize;
eig, temp: PIplImage;
quality, min_distance, dx, dy: double;
i: integer;
newpoint: PCvPoint2D32f;
rec: TRect;
begin
        if not (assigned(image )) then
        begin
            //* allocate all the buffers
            cs.width := frame.Width;
            cs.height := frame.Height;
            image := cvCreateImage( cs, 8, 3 );
            image.Origin := frame.origin;
            imageDiv := cvCreateImage( cvSize_(round(cs.width/2), round(cs.height/2)), 8, 3 );
            imageDiv.Origin := frame.origin;
            outImg := cvCreateImage( cs, 8, 3 );
            outImg.Origin := frame.origin;
        end;


        cvCopy(frame, image, 0);
        cvSmooth(image, image, CV_GAUSSIAN, 3, 3);

        cvCopy(image, outImg, 0);

        fFrameRecon.drawFrame(image, outImg);
        if cont=20 then
        begin
            cont := 1;
            IplImage2Bitmap(extractedFrame, bmp);
            rec := fFrameRecon.Image1.canvas.ClipRect;
            fFrameRecon.Image1.canvas.StretchDraw(rec , bmp);
            fFrameRecon.Image1.Repaint;
            Application.ProcessMessages;
        end else
                inc(cont);

        fFrameRecon.drawPerspective(outImg);
        cvCopy( outImg, frame, 0 );
end;


procedure call_back(frame: PIplImage); cdecl;
begin
                main_cycle(frame);
end;






procedure TfrmFrameRecon.FormCreate(Sender: TObject);
var
        nselCam, parm, n: longint;
        vf: VidFormat;
        outc: Plongint;
        whwnd: HWND;
        i, j: integer;

        moments: PCvMoments;

        contList, contours: PCvSeq;
        seqResult, hole, hole2: PCvContour;
        storage: PCvMemStorage;
        a, p: double;
        bRect: CvRect;
        maxA: double;
begin

    n := cvcamGetCamerasCount();
    nselCam := cvcamSelectCamera(outc);
    if (nselCam=0) then
    begin
        MessageDlg('No camera present!!', mtError, [mbOK], 0);
        halt;
    end;
    selcam := outc^;

    vf.height := 480;
    vf.width := 640;
    vf.framerate := 25;
    parm := cvcamtrue;
    cvcamSetProperty(selcam, CVCAM_PROP_ENABLE, @parm);
    cvcamSetProperty(selcam, CVCAM_PROP_RENDER, @parm);
    cvcamSetProperty(selcam, CVCAM_PROP_SETFORMAT, @vf);
    cvcamSetProperty(selcam, CVCAM_PROP_CALLBACK, @call_back);
    whwnd := fFrameRecon.panel1.Handle;
    cvcamSetProperty(selcam, CVCAM_PROP_WINDOW, @whwnd);
    MessageDlg('Keys: ESC - quit program', mtInformation, [mbOK], 0);

    bmp := TBitmap.Create;
    bmp.PixelFormat :=  pf24bit;



    intrinsic :=  cvLoad( 'Intrinsics.xml' );
    // multiply by 2 because the matrix is computed on 320x240 image
    cvConvertScale(intrinsic, intrinsic, 2.0, 0);


    distortion := cvLoad( 'Distortion.xml' );

    obj := cvCreateMat(4, 3, CV_32FC1);
    pro := cvCreateMat(4, 2, CV_32FC1);
    rot := cvCreateMat(1, 3, CV_32FC1);
    tra := cvCreateMat(1, 3, CV_32FC1);

    mapMatrix := cvCreateMat(3, 3, CV_32FC1);

    cvmSet(obj, 0, 0, -100);
    cvmSet(obj, 0, 1, -100);
    cvmSet(obj, 0, 2, 0.0);
    origQuad[0].x := 0;
    origQuad[0].y := 0;

    cvmSet(obj, 1, 0, 100.0);
    cvmSet(obj, 1, 1, -100);
    cvmSet(obj, 1, 2, 0.0);
    origQuad[1].x := 300;
    origQuad[1].y := 0;

    cvmSet(obj, 2, 0, 100.0);
    cvmSet(obj, 2, 1, 100.0);
    cvmSet(obj, 2, 2, 0.0);
    origQuad[2].x := 300;
    origQuad[2].y := 300;

    cvmSet(obj, 3, 0, -100);
    cvmSet(obj, 3, 1, 100.0);
    cvmSet(obj, 3, 2, 0.0);
    origQuad[3].x := 0;
    origQuad[3].y := 300;


    cvmSet(rot, 0, 0, 0.0);
    cvmSet(rot, 0, 1, 0.0);
    cvmSet(rot, 0, 2, 0.0);


    cvmSet(tra, 0, 0, 0.0);
    cvmSet(tra, 0, 1, 0.0);
    // the Z coord on translation vector is set to focal length
    cvmSet(tra, 0, 2, cvmGet(intrinsic, 0, 0));

    // appo image for warp perspective
    extractedFrame :=  cvCreateImage( cvSize_(800, 600), 8, 3 );


    cvcamInit;
    cvcamStart;

end;

procedure TfrmFrameRecon.FormKeyPress(Sender: TObject; var Key: Char);
begin
        if( key = char(27) ) then
        begin
            self.Destroy;
            halt;
        end;
end;

procedure TfrmFrameRecon.FormDestroy(Sender: TObject);
begin
    cvcamExit;
    cvReleaseImage(image);
    cvReleaseImage(outImg);
    cvReleaseImage(extractedFrame);
    bmp.Free;    
end;


{**********************************************************************}
{**********************************************************************}

procedure TfrmFrameRecon.btSettingsClick(Sender: TObject);
begin
    cvcamStop();
    cvReleaseImage(image);
    cvReleaseImage(outImg);
    cvcamGetProperty(selcam, CVCAM_VIDEOFORMAT, nil);
    cvcamGetProperty(selcam, CVCAM_CAMERAPROPS, nil);
    cvcamStart();

end;

procedure TfrmFrameRecon.extractContours(const origImg, image: PIplImage; const storage: PCvMemStorage;
                                var contList: PCvSeq);
var
    nc: integer;
    contours: PCvSeq;
    seqResult: PCvSeq;
    a: double;

begin
// *********** find contours in Canny result image

    // find contours and store them all as a list
   contours := nil;
    if (origImg.NChannels>1) then
    begin
        pgray := cvCreateImage( cvSize_(image.Width, image.Height) , IPL_DEPTH_8U, 1 );
        cvCvtColor( origImg, pgray, CV_BGR2GRAY )
    end else
        pgray := cvCloneImage(origImg);
    pcontor := cvCreateImage( cvSize_(image.Width, image.Height) , IPL_DEPTH_8U, 1 );
    if (cbCanny.Checked) then
            cvCanny( pgray, pcontor, sePar1.Value, sePar2.Value, StrToInt(cbPar3.Text) )
    else
            cvThreshold(pgray, pcontor, separ1.Value, 255, CV_THRESH_BINARY_INV);
    if (image.NChannels>1) and (cbDisplayCont.Checked) then
        cvCvtColor( pcontor, image, CV_GRAY2BGR );
// warning! findcontours destroy the input image, so display it before the call
    nc := cvFindContours( pcontor, storage, @contours, sizeof(CvContour),
        CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, cvPoint_(0,0) );

    if nc=0 then
        contList := nil
    else
        contLIst := contours;
    cvReleaseImage(pgray);
    cvReleaseImage(pcontor);
end;


procedure TfrmFrameRecon.drawFrame(const origImg, image: PIplImage);
var
    nc, i: integer;
    contList, contours: PCvSeq;
    seqResult, hole, hole2: PCvSeq;
    storage: PCvMemStorage;
    a, p: double;
    bRect : CvRect;

    nholes: integer;
    bol1, bol2, bol3: Boolean;
    m1, q1: single;
    m2, q2: single;
    vert1, vert2: Boolean;
    mi1, qi1: single;
    mi2, qi2: single;
    vertH1, vertH2: Boolean;

    p1, p2: PCvPoint;
    p1Ext: CvPoint;
    frame_found: Boolean;

    moments: PCvMoments;
    compactness: double;
begin
    storage := cvCreateMemStorage(0);
    extractcontours(origImg, image, storage, contList);

    // test each contour
    contours := contList;
    while( contours <> nil) do
    begin
        a :=  cvContourArea(contours, CV_WHOLE_SEQ);
        if (abs(a) > 500) and (contours.total>3) then
        begin
            seqResult :=  cvApproxPoly( contours, sizeof(CvContour), storage,
                         CV_POLY_APPROX_DP, sePolyPar1.Value, sePolyPar2.Value ) ;
            if seqResult.total=4 then
            begin
                frame_found := false;
//              calcola coefficienti delle rette diagonali del contorno esterno
                p1 := PCvPoint(cvGetSeqElem( seqresult, 0 ));
                p1Ext.x := p1.x;
                p1Ext.y := p1.y;
                p2 := PCvPoint(cvGetSeqElem( seqresult, 2 ));
                if (p1.x <> p2.x) then
                begin
                        vert1 := false;
                        m1 :=  (p2.y - p1.y) / (p2.x - p1.x);
                        q1 := (p2.x * p1.y - p1.x * p2.y) / (p2.x - p1.x);
                end else
                begin
                       vert1 := true;
                end;
                p1 := PCvPoint(cvGetSeqElem( seqresult, 1 ));
                p2 := PCvPoint(cvGetSeqElem( seqresult, 3 ));
                if (p1.x <> p2.x) then
                begin
                        vert2 := false;
                        m2 :=  (p2.y - p1.y) / (p2.x - p1.x);
                        q2 := (p2.x * p1.y - p1.x * p2.y) / (p2.x - p1.x);
                end else
                begin
                       vert2 := true;
                end;


               // internal contours with CV_RETR_TREE
                hole := contours.v_next;
                nholes := 0;
//              4) scorre i contorni interni (TREE)
                while (hole <> nil) do
                begin
                   inc(nholes);
                   if nholes>1 then break;     
                   if hole.total<=3 then Continue;     
//                 4.1) approssima ogni contorno con pilogono
                    hole2 := cvApproxPoly( hole, sizeof(CvContour), storage,
                               CV_POLY_APPROX_DP, sePolyPar1.Value, sePolyPar2.Value );
//                 4.2) verifica se poligono ha 4 vertici
                    if hole2.total<>4 then continue;
                    p1 := PCvPoint(cvGetSeqElem( hole2, 0 ));

//                 4.3) calcola coefficienti delle rette diagonali
                        p2 := PCvPoint(cvGetSeqElem( hole2, 2 ));
                        if (p1.x <> p2.x) then
                        begin
                                vertH1 := false;
                                mi1 :=  (p2.y - p1.y) / (p2.x - p1.x);
                                qi1 := (p2.x * p1.y - p1.x * p2.y) / (p2.x - p1.x);
                        end else
                        begin
                               vertH1 := true;
                        end;
                        p1 := PCvPoint(cvGetSeqElem( hole2, 1 ));
                        p2 := PCvPoint(cvGetSeqElem( hole2, 3 ));
                        if (p1.x <> p2.x) then
                        begin
                                vertH2 := false;
                                mi2 :=  (p2.y - p1.y) / (p2.x - p1.x);
                                qi2 := (p2.x * p1.y - p1.x * p2.y) / (p2.x - p1.x);
                        end else
                        begin
                               vertH2 := true;
                        end;


//                         4.4) se i coefficienti delle diagonali del poligono interno
//                              sono uguali a quelli del poligono esterno, a meno di errore dato,
//                              la ricerca e' positiva, uscire dalla funzione
                        if (vert1 = vertH1) and (vert2 = vertH2)
                            and (abs(m1 - mi1)/m1 <= (sePerimRatioDiff.Value / 100))
                            and (abs(q1 - qi1)/q1 <= (sePerimRatioDiff.Value / 100))
                            and (abs(m2 - mi2)/m2 <= (sePerimRatioDiff.Value / 100))
                            and (abs(q2 - qi2)/q2 <= (sePerimRatioDiff.Value / 100))   then
                        begin
                                    frame_found := true;
                                    Break;
                        end;

                        hole := hole.h_next;
                        
                end;    // while hole
            if (frame_found) then
            begin
                  cvZero(rot);
                  cvZero(tra);
                  // copy projection data to CvMat array
                  for i := 0 to 3 do
                  begin
                      p1 := PCvPoint(cvGetSeqElem( seqResult, i ));
                      cvmSet(pro, i, 0, p1.x);
                      cvmSet(pro, i, 1, p1.y);
                      quad[i].x := p1.x;
                      quad[i].y := p1.y;
                  end;


                  cvFindExtrinsicCameraParams2(obj, pro, intrinsic, distortion, rot, tra) ;
                  lbDist.Caption := 'Z dist. ' + FloatToStr(trunc(cvmGet(tra, 0, 2)));

                  cvGetPerspectiveTransform(@origquad, @Quad, mapMatrix);
                  bRect := pcvcontour(seqresult).rect;

                  cvzero(extractedFrame);
                  cvWarpPerspective(origImg, extractedFrame, mapMatrix,
                        (CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS+CV_WARP_INVERSE_MAP), cvScalar_(0,0,0,0));

                  cvDrawContours(image, PCvSeq(seqresult),CV_RGB(255,0,0),
                        CV_RGB(0,255,0),0,4,CV_AA,cvPoint_(0,0));

                  break;
            end;
        end;  // if seqresult.total=4
       end; // if abs
        // take the next contour
        contours := contours.h_next;
    end;
    cvRelease(@contList);
    cvReleaseMemStorage(storage);
end;




procedure TfrmFrameRecon.drawPerspective(image: PIplImage);
var
        mess: string;
        i,j : integer;
        pt1, pt2: CvPoint;
begin

        cvZero(pro);

        cvProjectPoints2(obj, rot, tra, intrinsic, distortion, pro);

        cvCircle(image, cvPoint_(round(cvmGet(pro, 0, 0)), round(cvmGet(pro, 0, 1))), 10,
                cv_rgb(255,0,0));
        for i := 0 to 3  do
        begin
                pt1.x := round(cvmGet(pro, i, 0));
                pt1.y := round(cvmGet(pro, i, 1));
                if i<3 then
                begin
                  pt2.x := round(cvmGet(pro, i+1, 0));
                  pt2.y := round(cvmGet(pro, i+1, 1));
                end else
                begin
                  pt2.x := round(cvmGet(pro, 0, 0));
                  pt2.y := round(cvmGet(pro, 0, 1));
                end;
                cvLine(image, pt1, pt2, CV_RGB(0,255,0),1, CV_AA,0);
        end;
end;


end.
